@@ -1,4 +1,3 @@
-from glob import glob
 from distutils import log
 import distutils.command.sdist as orig
 import os
@@ -6,14 +5,12 @@ import sys
 import io
 import contextlib
 
-from setuptools.extern import six
+import six, ordered_set
 
-from setuptools.utils import cs_path_exists
+from .py36compat import sdist_add_defaults
 
 import pkg_resources
 
-READMES = 'README', 'README.rst', 'README.txt'
-
 _default_revctrl = list
 
 
@@ -24,7 +21,7 @@ def walk_revctrl(dirname=''):
             yield item
 
 
-class sdist(orig.sdist):
+class sdist(sdist_add_defaults, orig.sdist):
     """Smart sdist that finds anything supported by revision control"""
 
     user_options = [
@@ -40,6 +37,9 @@ class sdist(orig.sdist):
 
     negative_opt = {}
 
+    README_EXTENSIONS = ['', '.rst', '.txt', '.md']
+    READMES = tuple('README{0}'.format(ext) for ext in README_EXTENSIONS)
+
     def run(self):
         self.run_command('egg_info')
         ei_cmd = self.get_finalized_command('egg_info')
@@ -51,13 +51,6 @@ class sdist(orig.sdist):
         for cmd_name in self.get_sub_commands():
             self.run_command(cmd_name)
 
-        # Call check_metadata only if no 'check' command
-        # (distutils <= 2.6)
-        import distutils.command
-
-        if 'check' not in distutils.command.__all__:
-            self.check_metadata()
-
         self.make_distribution()
 
         dist_files = getattr(self.distribution, 'dist_files', [])
@@ -90,8 +83,10 @@ class sdist(orig.sdist):
         """
         In a context, remove and restore os.link if it exists
         """
+
         class NoValue:
             pass
+
         orig_val = getattr(os, 'link', NoValue)
         try:
             del os.link
@@ -126,35 +121,8 @@ class sdist(orig.sdist):
     if has_leaky_handle:
         read_template = __read_template_hack
 
-    def add_defaults(self):
-        standards = [READMES,
-                     self.distribution.script_name]
-        for fn in standards:
-            if isinstance(fn, tuple):
-                alts = fn
-                got_it = 0
-                for fn in alts:
-                    if cs_path_exists(fn):
-                        got_it = 1
-                        self.filelist.append(fn)
-                        break
-
-                if not got_it:
-                    self.warn("standard file not found: should have one of " +
-                              ', '.join(alts))
-            else:
-                if cs_path_exists(fn):
-                    self.filelist.append(fn)
-                else:
-                    self.warn("standard file '%s' not found" % fn)
-
-        optional = ['test/test*.py', 'setup.cfg']
-        for pattern in optional:
-            files = list(filter(cs_path_exists, glob(pattern)))
-            if files:
-                self.filelist.extend(files)
-
-        # getting python files
+    def _add_defaults_python(self):
+        """getting python files"""
         if self.distribution.has_pure_modules():
             build_py = self.get_finalized_command('build_py')
             self.filelist.extend(build_py.get_source_files())
@@ -167,26 +135,23 @@ class sdist(orig.sdist):
                     self.filelist.extend([os.path.join(src_dir, filename)
                                           for filename in filenames])
 
-        if self.distribution.has_ext_modules():
-            build_ext = self.get_finalized_command('build_ext')
-            self.filelist.extend(build_ext.get_source_files())
-
-        if self.distribution.has_c_libraries():
-            build_clib = self.get_finalized_command('build_clib')
-            self.filelist.extend(build_clib.get_source_files())
-
-        if self.distribution.has_scripts():
-            build_scripts = self.get_finalized_command('build_scripts')
-            self.filelist.extend(build_scripts.get_source_files())
+    def _add_defaults_data_files(self):
+        try:
+            if six.PY2:
+                sdist_add_defaults._add_defaults_data_files(self)
+            else:
+                super()._add_defaults_data_files()
+        except TypeError:
+            log.warn("data_files contains unexpected objects")
 
     def check_readme(self):
-        for f in READMES:
+        for f in self.READMES:
             if os.path.exists(f):
                 return
         else:
             self.warn(
                 "standard file not found: should have one of " +
-                ', '.join(READMES)
+                ', '.join(self.READMES)
             )
 
     def make_release_tree(self, base_dir, files):
@@ -233,3 +198,34 @@ class sdist(orig.sdist):
                 continue
             self.filelist.append(line)
         manifest.close()
+
+    def check_license(self):
+        """Checks if license_file' or 'license_files' is configured and adds any
+        valid paths to 'self.filelist'.
+        """
+
+        files = ordered_set.OrderedSet()
+
+        opts = self.distribution.get_option_dict('metadata')
+
+        # ignore the source of the value
+        _, license_file = opts.get('license_file', (None, None))
+
+        if license_file is None:
+            log.debug("'license_file' option was not specified")
+        else:
+            files.add(license_file)
+
+        try:
+            files.update(self.distribution.metadata.license_files)
+        except TypeError:
+            log.warn("warning: 'license_files' option is malformed")
+
+        for f in files:
+            if not os.path.exists(f):
+                log.warn(
+                    "warning: Failed to find the configured license file '%s'",
+                    f)
+                files.remove(f)
+
+        self.filelist.extend(files)
